@@ -23,7 +23,7 @@
 #  Based on code from libpodcasts.py (thp, 2005-10-29)
 #
 
-import cgi
+import html
 import logging
 import os
 import re
@@ -54,8 +54,8 @@ class GEpisode(model.PodcastEpisode):
 
     @property
     def title_markup(self):
-        return '%s\n<small>%s</small>' % (cgi.escape(self.title),
-                          cgi.escape(self.channel.title))
+        return '%s\n<small>%s</small>' % (html.escape(self.title),
+                          html.escape(self.channel.title))
 
     @property
     def markup_new_episodes(self):
@@ -65,10 +65,10 @@ class GEpisode(model.PodcastEpisode):
             length_str = ''
         return ('<b>%s</b>\n<small>%s' + _('released %s') +
                 '; ' + _('from %s') + '</small>') % (
-                cgi.escape(re.sub('\s+', ' ', self.title)),
-                cgi.escape(length_str),
-                cgi.escape(self.pubdate_prop),
-                cgi.escape(re.sub('\s+', ' ', self.channel.title)))
+                html.escape(re.sub('\s+', ' ', self.title)),
+                html.escape(length_str),
+                html.escape(self.pubdate_prop),
+                html.escape(re.sub('\s+', ' ', self.channel.title)))
 
     @property
     def markup_delete_episodes(self):
@@ -83,11 +83,11 @@ class GEpisode(model.PodcastEpisode):
             downloaded_string = _('today')
         return ('<b>%s</b>\n<small>%s; %s; ' + _('downloaded %s') +
                 '; ' + _('from %s') + '</small>') % (
-                cgi.escape(self.title),
-                cgi.escape(util.format_filesize(self.file_size)),
-                cgi.escape(played_string),
-                cgi.escape(downloaded_string),
-                cgi.escape(self.channel.title))
+                html.escape(self.title),
+                html.escape(util.format_filesize(self.file_size)),
+                html.escape(played_string),
+                html.escape(downloaded_string),
+                html.escape(self.channel.title))
 
 
 class GPodcast(model.PodcastChannel):
@@ -131,8 +131,13 @@ class BackgroundUpdate(object):
                 (model.C_PUBLISHED, episode.published),
             )
             update_fields = model.get_update_fields(episode, include_description)
-            model.set(model.get_iter((self.index,)), *(x for fields in (base_fields, update_fields)
-                                                       for pair in fields for x in pair))
+            try:
+                it = model.get_iter((self.index,))
+            # fix #727 the tree might be invalid when trying to update so discard the exception
+            except ValueError:
+                break
+            model.set(it, *(x for fields in (base_fields, update_fields)
+                            for pair in fields for x in pair))
             self.index += 1
 
             # Check for the time limit of 20 ms after each 50 rows processed
@@ -188,7 +193,7 @@ class EpisodeListModel(Gtk.ListStore):
         self.ICON_IMAGE_FILE = 'image-x-generic'
         self.ICON_GENERIC_FILE = 'text-x-generic'
         self.ICON_DOWNLOADING = Gtk.STOCK_GO_DOWN
-        self.ICON_DELETED = Gtk.STOCK_DELETE
+        self.ICON_DELETED = 'edit-delete'
 
         self.background_update = None
         self.background_update_tag = None
@@ -274,20 +279,20 @@ class EpisodeListModel(Gtk.ListStore):
 
         if episode.state != gpodder.STATE_DELETED and episode.is_new:
             yield '<b>'
-            yield cgi.escape(title)
+            yield html.escape(title)
             yield '</b>'
         else:
-            yield cgi.escape(title)
+            yield html.escape(title)
 
         if include_description:
             yield '\n'
             if self._all_episodes_view:
-                yield _('from %s') % cgi.escape(episode.channel.title)
+                yield _('from %s') % html.escape(episode.channel.title)
             else:
                 description = episode.one_line_description()
                 if description.startswith(title):
                     description = description[len(title):].strip()
-                yield cgi.escape(description)
+                yield html.escape(description)
 
     def replace_from_channel(self, channel, include_description=False):
         """
@@ -497,6 +502,8 @@ class PodcastChannelProxy(object):
         self.cover_thumb = None
         self.auto_archive_episodes = False
 
+        self._update_error = None
+
     def get_statistics(self):
         # Get the total statistics for all channels from the database
         return self._db.get_podcast_statistics()
@@ -538,12 +545,15 @@ class PodcastListModel(Gtk.ListStore):
         self._max_image_side = 40
         self._cover_downloader = cover_downloader
 
-        self.ICON_DISABLED = 'gtk-media-pause'
+        self.ICON_DISABLED = 'media-playback-pause'
+        self.ICON_ERROR = 'dialog-warning'
 
     def _filter_visible_func(self, model, iter, misc):
+        channel = model.get_value(iter, self.C_CHANNEL)
+
         # If searching is active, set visibility based on search text
         if self._search_term is not None:
-            if model.get_value(iter, self.C_CHANNEL) == SectionMarker:
+            if channel == SectionMarker:
                 return True
             key = self._search_term.lower()
             columns = (model.get_value(iter, c) for c in self.SEARCH_COLUMNS)
@@ -551,6 +561,8 @@ class PodcastListModel(Gtk.ListStore):
 
         if model.get_value(iter, self.C_SEPARATOR):
             return True
+        elif getattr(channel, '_update_error', None) is not None:
+            return True
         elif self._view_mode == EpisodeListModel.VIEW_ALL:
             return model.get_value(iter, self.C_HAS_EPISODES)
         elif self._view_mode == EpisodeListModel.VIEW_UNDELETED:
@@ -687,36 +699,49 @@ class PodcastListModel(Gtk.ListStore):
         channel.cover_thumb = bytes(b''.join(bufs))
         channel.save()
 
-    def _get_cover_image(self, channel, add_overlay=False):
+    def _get_cover_image(self, channel, add_overlay=False, pixbuf_overlay=None):
+        """ get channel's cover image. Callable from gtk thread.
+            :param channel: channel model
+            :param bool add_overlay: True to add a pause/error overlay
+            :param GdkPixbuf.Pixbux pixbuf_overlay: existing pixbuf if already loaded, as an optimization
+            :return GdkPixbuf.Pixbux: channel's cover image as pixbuf
+        """
         if self._cover_downloader is None:
-            return None
+            return pixbuf_overlay
 
-        pixbuf_overlay = self._get_cached_thumb(channel)
+        if pixbuf_overlay is None:  # optimization: we can pass existing pixbuf
+            pixbuf_overlay = self._get_cached_thumb(channel)
 
         if pixbuf_overlay is None:
+            # load cover if it's not in cache
             pixbuf = self._cover_downloader.get_cover(channel, avoid_downloading=True)
             pixbuf_overlay = self._resize_pixbuf(channel.url, pixbuf)
             self._save_cached_thumb(channel, pixbuf_overlay)
 
-        if add_overlay and channel.pause_subscription:
-            pixbuf_overlay = self._overlay_pixbuf(pixbuf_overlay, self.ICON_DISABLED)
-            pixbuf_overlay.saturate_and_pixelate(pixbuf_overlay, 0.0, False)
+        if add_overlay:
+            if getattr(channel, '_update_error', None) is not None:
+                pixbuf_overlay = self._overlay_pixbuf(pixbuf_overlay, self.ICON_ERROR)
+            elif channel.pause_subscription:
+                pixbuf_overlay = self._overlay_pixbuf(pixbuf_overlay, self.ICON_DISABLED)
+                pixbuf_overlay.saturate_and_pixelate(pixbuf_overlay, 0.0, False)
 
         return pixbuf_overlay
 
     def _get_pill_image(self, channel, count_downloaded, count_unplayed):
         if count_unplayed > 0 or count_downloaded > 0:
-            return draw.draw_pill_pixbuf(str(count_unplayed), str(count_downloaded), widget=self.widget)
+            return draw.draw_pill_pixbuf('{:n}'.format(count_unplayed), '{:n}'.format(count_downloaded), widget=self.widget)
         else:
             return None
 
     def _format_description(self, channel, total, deleted,
             new, downloaded, unplayed):
-        title_markup = cgi.escape(channel.title)
-        if not channel.pause_subscription:
-            description_markup = cgi.escape(util.get_first_line(channel.description) or ' ')
+        title_markup = html.escape(channel.title)
+        if channel._update_error is not None:
+            description_markup = html.escape(_('ERROR: %s') % channel._update_error)
+        elif not channel.pause_subscription:
+            description_markup = html.escape(util.get_first_line(channel.description) or ' ')
         else:
-            description_markup = cgi.escape(_('Subscription paused'))
+            description_markup = html.escape(_('Subscription paused'))
         d = []
         if new:
             d.append('<span weight="bold">')
@@ -724,7 +749,9 @@ class PodcastListModel(Gtk.ListStore):
         if new:
             d.append('</span>')
 
-        if description_markup.strip():
+        if channel._update_error is not None:
+            return ''.join(d + ['\n', '<span weight="bold">', description_markup, '</span>'])
+        elif description_markup.strip():
             return ''.join(d + ['\n', '<small>', description_markup, '</small>'])
         else:
             return ''.join(d)
@@ -800,7 +827,7 @@ class PodcastListModel(Gtk.ListStore):
 
         for row in self:
             if row[self.C_URL] == url:
-                    return row.path
+                return row.path
         return None
 
     def update_first_row(self):
@@ -845,7 +872,7 @@ class PodcastListModel(Gtk.ListStore):
                     if isinstance(c, GPodcast) and c.section == section]
 
             # Calculate the stats over all podcasts of this section
-            if len(channels) is 0:
+            if len(channels) == 0:
                 total = deleted = new = downloaded = unplayed = 0
             else:
                 total, deleted, new, downloaded, unplayed = list(map(sum,
@@ -854,7 +881,7 @@ class PodcastListModel(Gtk.ListStore):
             # We could customized the section header here with the list
             # of channels and their stats (i.e. add some "new" indicator)
             description = '<span size="16000"> </span><b>%s</b>' % (
-                    cgi.escape(section))
+                    html.escape(section))
 
             self.set(
                 iter,
@@ -877,6 +904,7 @@ class PodcastListModel(Gtk.ListStore):
         self.set(iter,
                 self.C_TITLE, channel.title,
                 self.C_DESCRIPTION, description,
+                self.C_COVER, self._get_cover_image(channel, True),
                 self.C_SECTION, channel.section,
                 self.C_ERROR, self._format_error(channel),
                 self.C_PILL, pill_image,
@@ -902,9 +930,7 @@ class PodcastListModel(Gtk.ListStore):
         pixbuf = self._resize_pixbuf(channel.url, pixbuf)
         self._save_cached_thumb(channel, pixbuf)
 
-        if channel.pause_subscription:
-            pixbuf = self._overlay_pixbuf(pixbuf, self.ICON_DISABLED)
-            pixbuf.saturate_and_pixelate(pixbuf, 0.0, False)
+        pixbuf = self._get_cover_image(channel, add_overlay=True, pixbuf_overlay=pixbuf)
 
         for row in self:
             if row[self.C_URL] == channel.url:
